#!/usr/bin/python3
#
# Copyright [2023] Open Text.
#
# The only warranties for products and services of Open Text and its affiliates and licensors (“Open Text”)
# are as may be set forth in the express warranty statements accompanying such products and services.
# Nothing herein should be construed as constituting an additional warranty. Open Text shall not be liable
# for technical or editorial errors or omissions contained herein. The information contained herein is
# subject to change without notice.
#

# OES certificate Utility

import logging
import logging.handlers
import subprocess
import json
import configparser 
import os
#Common paths
BinaryPath = "/opt/novell/oes-cert-mgmt/bin"
eDirRSAPath = "/etc/ssl/servercerts/servercert.pem"
eDirECPath = "/etc/ssl/servercerts/serverECcert.pem"
ConfFile = "/etc/opt/novell/oes-cert-mgmt/oes-cert-mgmt.conf"
VarDataPath = "/var/opt/novell/oes-cert-mgmt"
ServiceCertsPath = "/var/opt/novell/oes-cert-mgmt/servicecerts"
CertificateLogFile = "/var/opt/novell/log/oes-cert-mgmt/oes-cert-mgmt.log"

#List modes
CERTIFICATE = 1
SERVICE = 2

#Expected return codes from service executables
FAIL = 200  #Operation failed
NOCERT = 201 #Service is not using any certificate
EDIRRSACERT= 202 #Service is using eDirectory RSA certificate /etc/ssl/servercerts/servercert.pem
EDIRECCERT= 203 #Service is using eDirectory ECDSA certificate /etc/ssl/servererts/serverECcert.pem
NONEDIRCERT= 204 #Service is not pointing to /etc/ssl/servercerts/*.pem

#Reconfiguration return code if any configuration operation is not supported
OPUNSUPPORTED = 205 #Unsupported operation

GETCERTBINARY = 1
RECONFIGBINARY = 2
OES_Services = ["Apache", "SFCB", "FTP", "iPrint", "NRM", "Postgres", "CIS-Configuration", "CIS-Core", "CIS-Infra", "CIS-Agent", "CIS-MariaDB", "CIS-CloudGateway", "Telemetry-Server", "Telemetry-Agent"]

#Get log level from conf file
def getloglevel():
    if not os.path.exists(ConfFile):
        print("Configuration file: " + str(ConfFile) + " Doesn't exist")
        return 10

    confparser = configparser.ConfigParser()
    confparser.read(ConfFile)
    loglevel = confparser.get('settings', "log-level")
    #print("log level from conf file: ", loglevel)
    if( loglevel != "CRITICAL" and loglevel != "ERROR" and loglevel != "WARNING" and loglevel != "INFO" and loglevel != "DEBUG"):
        return 10
    elif (loglevel == "CRITICAL"):
        return 50
    elif (loglevel == "ERROR"):
        return 40
    elif (loglevel == "WARNING"):
        return 30
    elif (loglevel == "INFO"):
        return 20
    else:
        return 10

logger = logging.getLogger('Logger')
logger.setLevel(logging.DEBUG)


def initialize_logger():
    """
    To initialize logger.
    """
        
    rfh = logging.handlers.RotatingFileHandler(CertificateLogFile, maxBytes=1024*1024*10, backupCount=2)
    formatter = logging.Formatter('%(asctime)s %(levelname)s:%(message)s')
    rfh.setFormatter(formatter)
    logger.addHandler(rfh)


def get_service_exec_path(servicename, scripttype):
    """
    Method to get service execution file path
    """

    path = ""
    if servicename in OES_Services:
        path = "/opt/novell/oes-cert-mgmt/bin/oes-cert-mgmt-" + servicename.lower()
        if scripttype == GETCERTBINARY:
            path = path + "-getcert"
        else:
            path = path + "-reconfig"
    else:
        print("Invalid service name for finding service executable path\n")

    return path


def run_command(cmd):
    """
    Function to execute linux command and capture the result
    :param cmd: linux command
    :return: Output of the command
    """
    err, output = None, None
    ret = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    try:
        (output, err) = ret.communicate()
    except Exception:
        ret.kill()

    if err:
        logger.error("Error while executing the command: " + str(cmd))
        logger.error("Error: " + str(err))
        return 0
    elif output:
        return output.decode('utf-8').strip()
    

def write_to_json_file(certObject, fileName):
    """
    To write to file.
    :return: return 1 on failure
    """
    jsonData=json.dumps(certObject, default=lambda o: o.__dict__, indent=4)
    fileWrite = None

    try:
        fileWrite = open(fileName, "w")
        fileWrite.write(jsonData)

    except Exception as e:
        logger.exception(e)
        logger.error("Failed to create the json file")
        return 1

    finally:
        if(fileWrite != None):
            fileWrite.close()




